from concurrent.futures import ThreadPoolExecutor
from typing import Dict, List, Optional

from loguru import logger

from src.core import api_request
from src.core.constant import PAY_RECORD_LIST_URL
from src.core.util import get_first_day_month
from src.entity.account import Account
from src.entity.member import MemberList, get_member_list
from src.entity.user import User


def get_pay_record_list(account: Account, date: str) -> Dict[str, List[str]]:
    logger.info(f'Getting pay record list for account: {account.name} and date: {date}')
    _names = {'name': account.name, 'names': []}
    params = {
        "pageNum": 1,
        "pageSize": 100,
        "registerSort": 1,
        "drawSort": -1,
        "depositSort": -1,
        "lastLoginTimeSort": -1,
        "name": "",
        "minPay": None,
        "maxPay": None,
        "startDate": get_first_day_month(),
        "registerStartDate": date,
        "endDate": date,
        "registerEndDate": date,
        "firstPayStartTime": "",
        "firstPayEndTime": "",
        "isBet": "0",
        "tagsFlag": "1"
    }
    member_list = get_member_list(account, params)
    if member_list is not None and len(member_list) > 0:
        with ThreadPoolExecutor(max_workers=len(member_list)) as executor:
            futures = [executor.submit(get_pay_record, account, member, date) for member in member_list]
            for future in futures:
                result = future.result()
                if result:
                    _names['names'].append(result)
    logger.info(f'Finished getting pay record list for account: {account.name} and date: {date}')
    return _names


def get_pay_record(account: Account, member: MemberList, date: str) -> Optional[str]:
    logger.info(f'Getting pay record for account: {account.name}, member: {member.name}, and date: {date}')
    params = {
        "pageNum": 1,
        "pageSize": 15,
        "id": member.id,
        "startDate": get_first_day_month(),
        "endDate": date
    }
    res = api_request.account_post(PAY_RECORD_LIST_URL, account=account, params=params)
    if int(res.data['orderAmountTotal']) > 0 and int(res.data['scoreAmountTotal']) == 0:
        return member.name
    return ""


def get_pay_failed_by_user(user: User, date: str) -> Optional[str]:
    logger.info(f'Getting pay failed by user: {user.username}')

    with ThreadPoolExecutor(max_workers=len(user.accounts)) as executor:
        futures = [executor.submit(get_pay_record_list, account, date) for account in user.accounts]

        # 使用列表推导式构建结果字符串
        text_lines = [
            "{}\n{}".format(res['name'], '\n'.join(res['names']))
            for future in futures if (res := future.result())['names']
        ]

    text = '\n'.join(text_lines)

    if not text:
        logger.info('无存款失败用户')
        return '无存款失败用户'

    logger.info(text)
    return text
