import asyncio
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from datetime import datetime

from src import logger
from src.core import api_request, util
from src.core.constant import PAY_RECORD_URL
from src.core.util import get_curr_day
from src.entity.account import Account
from src.entity.member import (async_get_member_detail_by_name,
                               get_member_by_name)


@dataclass
class PayRecord(object):
    billNo: str
    name: str
    agentName: str
    orderAmount: str
    rebateAmount: str
    flowRatio: str
    payType: int
    payName: str
    recipientAccount: str
    createdAt: str
    payStatus: int
    payStatusName: str
    whetherGetCard: str
    topId: int
    scoreAmount: str


# 根据用户查询最新存款信息
def get_pay_record_list(account: Account):
    logger.info(f'Getting pay record list for account: {account.name}')
    # 获取当前成功存款的用户
    params = {
        "memberName": "",
        "payState": 2,
        "isRest": False,
        "pageNum": 1,
        "pageSize": 100,
        "startDate": get_curr_day(),
        "endDate": get_curr_day()
    }
    api_response = api_request.account_post(PAY_RECORD_URL, account, params)
    logger.info(f'Finished getting pay record list for account: {account.name}')
    return [PayRecord(**item) for item in api_response.data['list']]


def get_latest_deposit_user(account: Account, count: int):
    logger.info(f'Getting latest deposit user for account: {account.name} and count: {count}')
    pay_record_list = get_pay_record_list(account)
    # 提取所有用户名
    names = []
    seen = set()
    now = datetime.now()
    for item in pay_record_list:
        name = item.name
        if name not in seen and util.get_difference(item.createdAt, now) < 7200:
            names.append(name)
            seen.add(name)
    logger.debug(f"获取到{len(names)}个成功存款人数")
    # 开启多线程 根据用户名查询所有数据
    with ThreadPoolExecutor(max_workers=len(names)) as executor:
        futures = [executor.submit(get_member_by_name, account, name) for name in names]
        try:
            results = [future.result() for future in futures]
        except Exception as e:
            logger.debug(f'查询失败：{e}')
        sorted_members = sorted(results, key=lambda member_detail: member_detail.get_first_pay_datetime(), reverse=True)
        # 截取前n个
        details = sorted_members[:count]
        record_dict = {record.createdAt: record.scoreAmount for record in pay_record_list}
        member_details = []
        for detail in details:
            if detail.firstPayAt in record_dict:
                detail.deposit = record_dict[detail.firstPayAt]
                member_details.append(detail)
        logger.info(f'Finished getting latest deposit user for account: {account.name} and count: {count}')
        return member_details


# 根据用户查询最新存款信息
async def async_get_pay_record_list(account: Account):
    logger.info(f'Async getting pay record list for account: {account.name}')
    today = get_curr_day()
    params = {
        "memberName": "",
        "payState": 2,
        "isRest": False,
        "pageNum": 1,
        "pageSize": 100,
        "startDate": today,
        "endDate": today
    }
    res = await api_request.async_account_post(PAY_RECORD_URL, account, params)
    logger.info(f'Finished async getting pay record list for account: {account.name}')
    return [PayRecord(**item) for item in res.data['list']]


# 获取最新存款用户
async def async_get_latest_deposit_user(account: Account, count: int):
    logger.info(f'Async getting latest deposit user for account: {account.name} and count: {count}')
    pay_record_list = await async_get_pay_record_list(account)
    # 提取所有用户名
    names = []
    seen = set()
    now = datetime.now()
    for item in pay_record_list:
        name = item.name
        # 存款订单有效期一般两个小时左右，所以默认只查询两个小时以内的订单即可
        if name not in seen and util.get_difference(item.createdAt, now) < 7200:
            names.append(name)
            seen.add(name)

    logger.debug(f"获取到{len(names)}个成功存款人数")
    tasks = []
    for name in names:
        task = asyncio.create_task(async_get_member_detail_by_name(account, name))
        tasks.append(task)

    results = await asyncio.gather(*tasks)
    # logger.info(f'查询成功:{results}')
    sorted_members = sorted(results, key=lambda member_detail: member_detail.get_first_pay_datetime(), reverse=True)
    # logger.debug(f'首存金额：{[member_detail.deposit for member_detail in sorted_members]}')
    # 因为会员列表的存款是统计用户当月的所有金额 所以有延迟 所以需要将订单中的金额赋值给存款金额 然后再返回
    details = sorted_members[:count]
    record_dict = {record.createdAt: record.scoreAmount for record in pay_record_list}
    for detail in details:
        if detail.firstPayAt in record_dict:
            detail.deposit = record_dict[detail.firstPayAt]
    logger.info(f'Finished async getting latest deposit user for account: {account.name} and count: {count}')
    return details
